#! /usr/bin/env python
# -*- coding: utf-8 -*-
# file: Figures.py
#
# Copyright (c) 2016 Ahmed Omran <ahmed.omran@mpq.mpg.de>

"""
Standard module for creating and formatting matplotlib figures.
"""

import numpy as np
from matplotlib import rcParams
from matplotlib import rc
import matplotlib.pyplot as plt


### Standardised symbol style for data points ###
symbolStyle = {'marker':'o',
               'markersize':5,
               'markeredgewidth':0.8,
               'linestyle':'None'}

symbolStyleInset = {'marker':'o',
               'markersize':2,
               'markeredgewidth':0.8,
               'linestyle':'None'}


def setMPLParams(usetex=False):
    """
    Sets default matplotlib parameters

    Parameters
    ----------
    usetex : bool
           Flag for rendering text with LaTeX

    Notes
    -------
    Latex should not be used if you do not intend to typeset a formula into
    your figure.
    
    If usetex is False and a formula is included into the figure, the font
    defaults to Computer Modern.  For switching to other fonts such as Times or
    Palatino A preamble entry is necessary, which only works if usetex is set
    to True.

    However, if usetex is True, all tick labels and axis labels are typeset in
    LaTeX as well, which makes it more difficult to change their font
    properties. In particular, Helvetica Neue doesn't work if latex or
    pdflatex, which forces you to use standard Helvetica instead. 

    If you insist on having full font flexibility with latex included, you can
    use xelatex or lualatex for typesetting, which gives much more flexibility
    in setting the font properties. But then, things start to get messy.
    """
    
    if usetex:
        rc('text', usetex=True)
        rcParams['font.sans-serif'] = 'Helvetica'
        rcParams['text.latex.preamble'] = r'\usepackage{mathptmx}'
  
    else:
        rcParams['font.family'] = 'sans-serif'
        rcParams['font.sans-serif'] = ['Helvetica Neue']

    
    rcParams.update({'axes.linewidth':0.8,
        'ytick.major.size':4.,
        'xtick.major.size':4.,
        'xtick.major.width':0.8,
        'ytick.major.width':0.8,
        'xtick.minor.size':2.,
        'ytick.minor.size':2.,
        'xtick.minor.width':0.8,
        'ytick.minor.width':0.8,
        'font.size':'10',
        'axes.unicode_minus':False})
    


def newfig(scale, height_ratio = 0.65, text_width = 450.):
    """
    Create a new figure with a given size

    Parameters
    ----------
    scale : float
          Figure width relative to text column

    height_ratio : float
                 Height ratio relative to figure width

    text_width : float
               Text column width in pt

    Returns
    -------
    fig : Figure
        The figure container

    ax : Axes
       Axes container that fills the figure canvas
    """

    width_pt = text_width                 # Text column width in pt
    width_inch = (1./72.27) * width_pt    # Convert pt to inch
    fig_width = width_inch * scale    # Width in inches

    fig_height = fig_width * height_ratio                # Height in inches
    
    fig = plt.figure(figsize = [fig_width, fig_height])
    ax = fig.add_subplot(111)
    return fig, ax



def savefig(filename,tight=False):
    """
    Saves current figure as PDF

    Parameters
    ----------
    filename : string
             Name of saved PDF

    tight : bool
          Flag for setting tight bounding box
    """

    if tight:
        plt.savefig('{}.pdf'.format(filename),bbox_inches='tight')
    else:
        plt.savefig('{}.pdf'.format(filename))



def makeRoundTicks(ax):
    """
    Changes capstyle of all tick marks to round

    Parameters
    ----------
    ax : Axes
       Axes container of interest

    Notes
    -----
    This feature works so far in matplotlib 1.5, however there are
    apparently plans to break this in later versions.
    
    Refer to http://stackoverflow.com/questions/32539867
    """
    for i in ax.xaxis.get_majorticklines():
        i._marker._capstyle = 'round'
    for i in ax.xaxis.get_minorticklines():
        i._marker._capstyle = 'round'
    for i in ax.yaxis.get_majorticklines():
        i._marker._capstyle = 'round'
    for i in ax.yaxis.get_minorticklines():
        i._marker._capstyle = 'round'



def changeTickFonts(ax):
    """
    Changes font of tick labels 

    Parameters
    ----------
    ax : Axes
       Axes container of interest

    Notes
    -----
    This feature works up to matplotlib 1.5 without any trouble, but there are
    apparently plans to break this in later versions.
    
    For more info, refer to http://stackoverflow.com/questions/32539867
    """
    font = {'family':'Helvetica Neue',
            'style':'normal',
            'weight':'normal',
            'stretch':'normal'}

    ax.set_xticklabels(ax.get_xticks(), **font)
    ax.set_yticklabels(ax.get_yticks(), **font)



def makeRoundErrorBars(ax,xdata,ydata,yerr,linecolor,linewidth=0.8):
    """
    Draw error bars with round caps

    Parameters
    ----------
    ax : Axes
       Axes container of interest

    xdata : list or array
          Argument values

    ydata : list or array
          Data values

    yerr : list or array
         Standard deviation errors of the data values

    linecolor : string
              Matplotlib color or HTML representation '#0123AE'

    linewidth : float
              Width of error bar lines

    Notes
    -----
    Could not find a way to change the cap style of error bars. This is a
    simple workaround that draws the error bar lines as simple lines going from
    y - sigma_y to y + sigma_y for each x value.

    TODO: Include support for asymmetric error bars
    """

    for i in range(len(xdata)):
        ax.plot([xdata[i], xdata[i]],
                [ydata[i] + yerr[i],
                ydata[i]-yerr[i]],
                solid_capstyle='round',
                color=linecolor, lw=linewidth)
